using System;
using System.Threading.Tasks;
using System.Collections.Generic;
using System.Linq;
using Aurora.Services;
using Aurora.Proto.Events;
using Aurora.Proto.General;

namespace Aurora.RemoteImpl
{
    public class RemoteEventServiceImpl : RemoteEventService.RemoteEventServiceBase
    {
        public RemoteEventServiceImpl()
        {

        }

        /// <summary>
        /// RPC for getting event stream for a particular client.
        /// </summary>
        /// <param name="request">Empty</param>
        /// <param name="responseStream">The response stream</param>
        /// <param name="context">gRPC client context</param>
        /// <returns></returns>
        public async override Task GetEvents(EventsRequest request, Grpc.Core.IServerStreamWriter<BaseEvent> responseStream, Grpc.Core.ServerCallContext context)
        {
            try
            {
                while (EventManager.Instance.GetSubscriptionCount(Combine(new string[] { context.Peer, request.ClientId })) > 0)
                {
                    Console.WriteLine("Peer " + context.Peer);
                    //TODO this causes crashes when two or more members are connected
                    //TODO Change this to events based stream instead of a poll based...
                    List<BaseEvent> events = EventManager.Instance.GetSessionEvents(Combine(new string[] { context.Peer, request.ClientId }));
                    foreach (BaseEvent currentEvent in events)
                    {
                        await responseStream.WriteAsync(currentEvent);
                    }
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception caught:" + ex.Message);
            }
        }

        /// <summary>
        /// RPC for subscribing to remote events
        /// </summary>
        /// <param name="request">SubscribeRequest</param>
        /// <param name="context">gRPC client context</param>
        /// <returns></returns>
        public override Task<SubscriptionResponse> SubscribeToEvents(SubscribeRequest request, Grpc.Core.ServerCallContext context)
        {
            EventManager.Instance.AddSubscriptionList(Combine(new string[] { context.Peer, request.ClientId }), request.EventTypes.ToList());

            return Task.FromResult(new SubscriptionResponse { Successful = true });
        }

        /// <summary>
        /// RPC for unsubscibing from events
        /// </summary>
        /// <param name="request">UnsubscribeRequest</param>
        /// <param name="context">gRPC client context</param>
        /// <returns></returns>
        public override Task<SubscriptionResponse> UnsubscribeFromEvents(UnsubscribeRequest request, Grpc.Core.ServerCallContext context)
        {
            EventType[] eventTypes = null;
            request.EventTypes.CopyTo(eventTypes, 0);

            EventManager.Instance.RemoveSubscriptionList(Combine(new string[] { context.Peer, request.ClientId }), eventTypes.ToList());

            return Task.FromResult(new SubscriptionResponse { Successful = true });
        }

        /// <summary>
        /// RPC for unsubscribing from all events
        /// </summary>
        /// <param name="request">UnsubscribeAllRequest</param>
        /// <param name="context">gRPC client context</param>
        /// <returns></returns>
        public override Task<SubscriptionResponse> UnsubscribeFromAll(UnsubscribeAllRequest request, Grpc.Core.ServerCallContext context)
        {
            EventManager.Instance.RemoveAllSubscriptions(Combine(new string[] { context.Peer, request.ClientId }));

            return Task.FromResult(new SubscriptionResponse { Successful = true });
        }

        private string Combine(string[] args)
        {
            string outString = "";
            foreach (string arg in args)
            {
                if (arg == args.Last())
                {
                    outString += arg;
                }
                else
                {
                    outString += arg + ":";
                }
            }

            return outString;
        }
    }

}